/****************************************************************************
** CHRISTOPHER KRATZ - CS460 - TOS DATA ENGINE CLASS IMPLEMENTATION  **
****************************************************************************/

#include"DataServer.h"
#include<string.h>




/********************************************************/
/********************************************************/
/*			MESSAGING SYSTEM FUNCTIONS                  */
/*######################################################*/
/*                         __        .____              */
/*                        | |        |    '             */
/*                     .-----.-------|    |-------.     */
/*       _           ./  | |  \.     | +--"        \.   */
/*      | |         /           \    | |             \  */
/*      | | __      |           |:   | |             |  */
/* -----+ ":__|     |  ~~~~~~~  |:   | |             |  */
/*         :__|     |           |:  .| |.            |  */
/*         :__|     |           |:  | * |            |  */
/* --------:__|     | US MAIL   |:  |___|            |  */
/*                  |           |:                   |  */
/*                  |___________|:                   |  */
/*	                 ~~~~~~~~~~~'""""""""""""""""""""'  */
/*	                                  |=.|.:|           */
/*	                                  |=.|.:|           */
/*                                    |=.|.:|           */
/*                                    |=.|.:|           */
/*                                                      */
/*######################################################*/

cDataEngine::cDataEngine()
{
	//Code to load configuration should go here

	Queues = NULL;
	intQueueCnt = 0;
	
	/* This code is temporary for testing, eventually the system will load
		its configuration from a file dynamically			*/
	AddQueue("Agent1", SIZE_QUEUE);
	AddQueue("Agent2", SIZE_QUEUE);

}

cDataEngine::~cDataEngine()
{
	//Deconstructor
	
}


bool cDataEngine::AddQueue(char * name, int size)
{
	/*	This function adds a new queue to the Queues array.
		This would occur when adding a new client to the system, 
		probably during initializaton			*/
		
	cQueue * newQ = new cQueue(name, size);
	if(newQ == NULL) return false;
		
	int i;
	
	cQueue ** newQlist = new cQueue * [intQueueCnt + 1];
	if(newQlist == NULL) return false;
	

	for(i=0; i<intQueueCnt; i++)
	{
		newQlist[i] = Queues[i];
	}
	newQlist[i] = newQ;
	intQueueCnt++;
	
	delete[] Queues;
	Queues = newQlist;
	
	return true;
}

void cDataEngine::RemoveQueue(char * name)
{
	//Remove a queue from the system
}

void cDataEngine::LockQueue(char * name)
{
	//Locking to prevent multiple access
}

void cDataEngine::ReleaseQueue(char * name)
{
	//release a locked queue
}


int cDataEngine::GetMess(char * name, char * from, char * data,  int& length, int& ServiceNumber)
{
	/* Find the named queues index */
	int i;
	for(i = 0; i<intQueueCnt; i++)
	{
		if(strcmp(name, Queues[i]->name)==0)
		{
			break;
		}
	}
	int q = i;
	if(i==intQueueCnt)
	{
		return -2;
	}

	/* Get the next message off the queue */
	cMessage mess;
	if(Queues[q]->RemoveMessage(mess)==false)
	{
		/*Queue is empty*/
		return -1;
	}
	
	//data = new char[mess.intSize];
	
	/* Set return values */
	length =  mess.intSize;
		
	/* Copy data */
	for(i=0; i<length; i++)
	{
		data[i] = mess.data[i];
	}

	/* Free Memory */
	delete[] mess.data;

	/* Copy Destination */
	strncpy(from, mess.sender, SIZE_AGENTNAME);
	/* Free memory */
	delete[] mess.sender;

	/* Copy ServiceNumber */
	ServiceNumber = mess.ServiceNumber;

	/* Return the number of messages left in queue */
	return Queues[q]->getLength();
	
}

bool cDataEngine::PostMess(char * name, char * from, char * data,  int length, int ServiceNumber)
{
	/* Find the named queues index */
	int i;
	for(i = 0; i<intQueueCnt; i++)
	{
		if(strcmp(name, Queues[i]->name)==0)
		{
			break;
		}
	}
	int q=i;
	if(i==intQueueCnt)
	{
		return false;
	}
	/*---------------------------*/
	

	/* Create New Message */
	cMessage mess;
	mess.intSize = length;
	
	/* Allocate memory */
	mess.data = new char[length];
	/* Copy data into mess.data */
	for(i=0; i<length; i++)
	{
		mess.data[i] = data[i];
	}

	/* Allocate memory and copy from to mess.sender */
	mess.sender = new char[SIZE_AGENTNAME];
	strncpy(mess.sender, from, SIZE_AGENTNAME);

	/* Set Service Number */
	mess.ServiceNumber = ServiceNumber;
	
	/*Add The message */
	Queues[q]->AddMessage(mess);
	
	return true;
	
}


cQueue::cQueue(char * qname, int size)
{
	strcpy(name, qname);
	intSize = size;
	intFront = 0;
	intBack = 0;
	Messages = new cMessage[intSize];
	
}

cQueue::~cQueue()
{
	delete[] Messages;
}

int cQueue::getLength()
{
	if(intBack < intFront)
	{
		return intFront - intBack;
	}
	else if(intFront < intBack)
	{
		return (intSize - intBack) + intFront;
	}
	else
	{
		return 0;
	}

}

bool cQueue::AddMessage(cMessage mess)
{
	int old = intFront;
	
	intFront++;
	if(intFront==intSize)
	{
		intFront=0;
	}
	//Check queue for overflow
	if(intFront==intBack)
	{
		//Queue is full, drop message, 
		intFront = old;
		return false;
	}
	
	Messages[intFront] = mess;

	return true;
}

bool cQueue::RemoveMessage(cMessage& mess)
{
	
	if(intBack==intFront)
	{
		return false;
	}
	intBack++;
	if(intBack==intSize)
	{
		intBack=0;
	}

	mess = Messages[intBack];
	
	return true;
}
	
/********************************************************/
/********************************************************/
/*			MESSAGING SYSTEM FUNCTIONS                  */
/*######################################################*/

/********************************************************************/
/********************************************************************/
/* DATA TABLE FUNCTIONS												*/
/********************************************************************/
/* DATA TABLE IMPLEMENTATIONS       _--"--_                         */
/*     ._______________           ."       ".                       */
/*     | ._____________|__  ------|"-,,_,,-"|                       */
/*     |.| .______________|__.   .|         |                       */
/*     |.|.|....:....:....:..| ./ |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|/   |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|    |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|    |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|     "-,,_,,-"                        */
/*     "-|.|####:####:####:##|                                      */
/*       |.|XXXX:XXXX:XXXX:XX|                                      */
/*       "-|    :    :    :  |                                      */
/********************************************************************/

cDataTable::cDataTable(unsigned long rows, unsigned long cols)	/* CONSTRUCTOR */
{
	/* initialize table */
	intRows = rows;			/* MAX NUMBER ROWS                */	
	intCols = cols;			/* MAX NUMBER COLUMNS             */
	intRowWidth = 0;		/* UPDATED WHEN COLUMNS ARE ADDED */	
	intInsertRowPos = 0;	/* INSERTIONS START AT ROW 0	  */
	intInsertColPos = 0;	/* INSERTIONS START AT ROW 0	  */
	intFilledRows	= 0;	/* NUMBER OF USED ROWS	          */
	intFilledCols	= 0;	/* NUMBER OF USED ROWS	          */

	Columns = new cDataColumn[intCols];
	if( Columns == NULL )
		return;

	Rows = new void *[intRows];
	if( Rows == NULL)
	{
		delete Columns;
		return;
	}
	/* INIT ALL ROWS TO NULL */
	unsigned long i;
	for(i=0;i<rows;i++)
	{
		Rows[i] = NULL;
	}
	
	return;	

}

cDataTable::~cDataTable()
{
	delete[] Columns;

	unsigned long i;
	for(i=0; i<intRows; i++)
	{
		delete Rows[i];
	}
	delete[] Rows;
}

void * cDataTable::NewRow()
{
	char * newrow = new char[intRowWidth];
	if(newrow==NULL) return NULL;

	unsigned int i;
	for(i=0;i<intRowWidth;i++)
	{
		newrow[i] = 0x00;
	}

	return (void *)newrow;
}

bool cDataTable::AddRow(void * row)
{
	if(row == NULL) 
		return false;

	if((intFilledRows + 1) >= intRows)
	{
		/* TABLE FULL */
		return false;
	}
	
	/* FIND FIRST EMPTY SPOT, SHOULD BE CURRENT */
	while(Rows[intInsertRowPos] != NULL)
	{
		intInsertRowPos++;
		if(intInsertRowPos >= intRows)
			intInsertRowPos = 0;
	}
	Rows[intInsertRowPos] = row;
	
	return true;
}

bool cDataTable::DeleteRow(unsigned long index)
{
	if(index >= intRows)
		return false;

	delete Rows[index];
	Rows[index] = NULL;

	return true;
}

LIST * cDataTable::FindRows(unsigned long col, void * val)
{
	if(col == 0xffffffff || val == NULL || col >= intCols)
		return NULL;

	
	unsigned long startbyte = Columns[col].start;
	unsigned long size	    = Columns[col].size;
	bool indexed = Columns[col].indexed;

	/*START LINKED LIST*/
	LIST * root = new LIST;
	root->value = 0xffffffff;
	root->link = NULL;
	

	if(root == NULL)
		return NULL;

	LIST * retpos = root;

	if(indexed)
	{
		return NULL;
	}
	else
	{
		/* LINEAR SEARCH */
		unsigned long i,j;
		for(i=0;i<intRows;i++)
		{
			unsigned long match = 0;
			for(j=0;j<size;j++)
			{
				if(  ((char*)(Rows[i]))[j+startbyte] ==  (char)((char *)val)[j]  )
				{
					match++;	
				}
				else
				{
					break;
				}
			}
			if(match == size)/* All bytes match */
			{
				/* ADD RESULT TO LINKED LIST*/
				retpos->link = new LIST;
				retpos = retpos->link;
				if(retpos==NULL)
				{
					DestroyList(root);
					return false;
				}
				retpos->link = NULL;
				retpos->value = i;
				return root;
			}
		}
	}

	return root;
}
	

LIST * cDataTable::FindRows(char * col, void * val)
{	
	unsigned long index = GetColumnIndex(col);
	if(index == 0xffffffff)
		return NULL;

	return FindRows(index, val);
}

void cDataTable::DestroyList(LIST * root)
{
	if(root->link == NULL)
		return;

	DestroyList(root->link);

	delete root;
}
	

bool cDataTable::UpdateRow(void * row, unsigned long index)
{
	if(index == 0xffffffff || index >= intRows || row == NULL)
		return false;

	delete Rows[index];
	Rows[index] = row;

	return true;
}

cDataColumn cDataTable::NewColumn(unsigned long size,
									unsigned long type,
									bool indexed,
									char * name)
{
	cDataColumn newcol;
	newcol.start = 0xffffffff;
	newcol.size  = size;
	newcol.type  = type;
	newcol.indexed = indexed;
	strncpy(newcol.name, name, SIZE_COLUMNNAME);

	return newcol;
}

bool cDataTable::AddColumn(cDataColumn col)
{
	
	if(intInsertColPos >= intCols)
		return false;

	col.start = intRowWidth;
	intRowWidth += col.size;
	
	Columns[intInsertColPos] = col;
	intInsertColPos++;
	intFilledCols++;
	return true;
}

unsigned long cDataTable::GetColumnIndex(char * col)
{
	unsigned long i;
	for(i=0;i<intCols;i++)
	{
		if( strncmp(col, Columns[i].name, SIZE_COLUMNNAME)==0)
		{
			return i;
		}
	}
	return 0xffffffff; /* not found */
}

bool cDataTable::EditRow(void * row, unsigned long col, void * val)
{
	if(row==NULL || col >= intFilledCols || val==NULL || col==0xffffffff)
		return false;

	unsigned long startbyte = Columns[col].start;
	unsigned long size	    = Columns[col].size;
	
	unsigned long i;
	for(i=0;i<size;i++)
	{
		((char *)row)[i + startbyte] = ((char *)val)[i];
	}

	return true;
}

bool cDataTable::EditRow(void * row, char * col, void * val)
{
	unsigned int index = GetColumnIndex(col);
	if(index==0xffffffff)
		return false;

	return EditRow(row, index, val);

}

bool cDataTable::GetColValue(void * row, unsigned long col, void * val)
{
	if(row==NULL || col >= intFilledCols || val==NULL || col==0xffffffff)
		return false;

	unsigned long startbyte = Columns[col].start;
	unsigned long size	    = Columns[col].size;
	
	unsigned long i;
	for(i=0;i<size;i++)
	{
		((char *)val)[i] = ((char *)row)[i + startbyte];
	}

	return true;
}

bool cDataTable::GetColValue(void * row, char * col, void * val)
{
	unsigned int index = GetColumnIndex(col);
	if(index==0xffffffff)
		return false;

	return GetColValue(row, index, val);

}

void * cDataTable::GetRow(unsigned long index)
{
	if(index==0xffffffff)
		return NULL;

	return Rows[index];
}


/*####################################################################################*/
/*                                                                                    */
/*    %%%%  %%%%   %%%%%%%    %%%%%%%%%  %%%%%%%%%  %%%%%%%%%                         */
/*    %%%%  %%%%   %%%%%%%%   %%%   %%%  %%%%%%%%%  %%%%%%%%%                         */
/*   %%%%%%%%     %%%%  %%%  %%%%%%%%%     %%%        %%%%%                           */
/*   %%%%%%%%     %%%%%%%%   %%%%%%%%%     %%%      %%%%                              */
/*  %%%%  %%%%   %%%% %%%   %%%   %%%     %%%     %%%%%%%%%                           */
/*  %%%%  %%%%   %%%  %%%%  %%%   %%%     %%%     %%%%%%%%%                           */
/*                                                                                    */
/*      #######   ####    ####    ########       ****     ***     ***    ******       */
/*     #########  #####   ####   ##########     ******   *****   *****   *******      */
/*    ####        ######  ####  ####    ###          *  **   ** **   **       **      */
/*    ####        ######  ####  ####               **   **   ** **   **      **       */
/*     #######    ####### ####  ####              **    **   ** **   **     **        */
/*      #######   ############  ####            ******   *****   *****     **         */
/*          ####  #### #######  ####            ******    ***     ***     **          */
/*          ####  ####  ######  ####    ###                                           */
/*    #########   ####   #####   ##########      -- ---  !  ! !    +--+               */
/*     #######    ####    ####    ########      |   ",   +--! !--+ |  |               */
/*                                               -- --"     ! !__! +--+               */
/*####################################################################################*/

	