/************************************************************************
** CHRISTOPHER KRATZ - CS460 - TOS DATA ENGINE CLASS DEFINITIONS  **
************************************************************************/

/*MESSAGINGE*/
const SIZE_QUEUENAME	= 255;
const SIZE_MESSAGE		= 255;
const SIZE_QUEUE		= 255;

/* DATA */
const SIZE_COLUMNNAME	= 255;


/* BOTH */
const SIZE_AGENTNAME	= 255;
const SIZE_PROCESSNAME	= 255;
const SIZE_PIPENAME		= 255;







/********************************************************************/
/* DATA TABLE IMPLEMENTATIONS       _--"--_                         */
/*     ._______________           ."       ".                       */
/*     | ._____________|__  ------|"-,,_,,-"|                       */
/*     |.| .______________|__.   .|         |                       */
/*     |.|.|....:....:....:..| ./ |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|/   |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|    |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|    |         |                       */
/*     |.|.|XXXX:XXXX:XXXX:XX|     "-,,_,,-"                        */
/*     "-|.|####:####:####:##|                                      */
/*       |.|XXXX:XXXX:XXXX:XX|                                      */
/*       "-|    :    :    :  |                                      */
/********************************************************************/

struct LIST
{
	unsigned long value;
	LIST * link;
};

/* DEFINES THE COLUMNS THAT EXIST IN A TABLE */
struct cDataColumn
{
	unsigned long start;		/* START OF FIELD                  */
	unsigned long size;			/* SIZE OF FIELD                   */
	unsigned long type;			/* OPTIONAL TYPE INDICATOR         */
	char name[SIZE_COLUMNNAME];	/* OPTIONAL TEXT NAME OF THE TABLE */
	bool indexed;				/* IS THE COLUMN INDEXED		   */
};

/* DEFINES THE DATA TABLE ITSELF*/
class cDataTable
{
private:

	unsigned long intRows;			/* NUMBER OF ROWS IN THE TABLE    */
	unsigned long intCols;			/* NUMBER OF COLUMNS IN THE TABLE */
	unsigned long intRowWidth;		/* NUMBER OF BYTES ROW IS WIDE    */
	unsigned long intInsertRowPos;	/* NEXT ROW WILL BE INSERTED HERE */
	unsigned long intInsertColPos;	/* NEXT COL WILL BE INSERTED HERE */
	unsigned long intFilledRows;	/* NUMBER OF USED ROWS IN TABLE	  */
	unsigned long intFilledCols;	/* NUMBER OF USED COLS IN TABLE	  */


	cDataColumn * Columns;		/* AN ARRAY OF COLUMN DEFINITIONS	 */
	void		** Rows;		/* AN ARRAY OF POINTERS TO DATA ROWS */

public:
	cDataTable(unsigned long rows, unsigned long cols);	/* CONSTRUCTOR */
	~cDataTable();												/* DESTRUCTOR  */

	/* DATA ROW FUNCTIONS */
	void * NewRow();										/* ALLOCATES MEMORY FOR A NEW DATA ROW AND RETURNS POINTER */
	bool AddRow(void * row);								/* INSERTS ROW INTO AN EMPTY SLOT    */
	bool DeleteRow(unsigned long index);					/* DELETES A ROW AT A SPECIFIC INDEX */	
	bool EditRow(void * row, unsigned long col, void * val);/* CHANGES COL VALUE IN ROW */
	bool EditRow(void * row, char * col, void * val);		/* CHANGES COL VALUE IN ROW */
	bool GetColValue(void * row, unsigned long col, void * val);/* GETS COL VALUE IN ROW */
	bool GetColValue(void * row, char * col, void * val);/* GETS COL VALUE IN ROW */
	void * GetRow(unsigned long index);						/* GETS A ROW BY INDEX */

	LIST * FindRows(char * col, void * val);				/* MATCHES CRITERIA AND RETURNS POINTER TO AN ARRAY OF INDEXES */
	LIST * FindRows(unsigned long col, void * val);			/* MATCHES CRITERIA AND RETURNS POINTER TO AN ARRAY OF INDEXES */

	void DestroyList(LIST * root);						/* Disposes of dynamic linked list */

	bool UpdateRow(void * row, unsigned long index);		/* REPLACES ROW AT INDEX WITH DATA */
	//bool UpdateRow(unsigned long * cols, void ** data, unsigned long cnt);		/* REPLACES SPECIFIED COLUMNS WITH NEW DATA */
	
	/* DATA COLUMN FUNCTIONS */
	unsigned long GetColumnIndex(char * col);		/* GETS THE INDEX NUMBER OF THE NAMED COLUMN */
	cDataColumn   NewColumn(unsigned long size,		/* CREATES A NEW DATA COLUMN DEFINITION */
							unsigned long type,
							bool indexed,
							char * name );
	bool AddColumn(cDataColumn  col);				/* INSERTS A NEW COLUMN IN ARRAY */

};

/************************************************/
/***** END DATA STUFF ***************************/
/************************************************/

struct cMessage
{
	int intSize;
	char * data;
	char * sender;
	int ServiceNumber;
};

class cQueue
{
private:
	int intSize;
	
	int intFront;
	int intBack;
	cMessage * Messages;
	
public:
	
	char name[SIZE_QUEUENAME];
	bool AddMessage(cMessage mess);
	bool RemoveMessage(cMessage& mess);
	int getLength();
	cMessage * GetMessage();
	
	cQueue(char * name, int size);
	~cQueue();
};

struct cClient
{
	char pipename[SIZE_PIPENAME];
	char agentname[SIZE_AGENTNAME];
	bool bServerRun;
	char processname[SIZE_PROCESSNAME];
};



class cDataEngine
{
private:
	//int intTableCnt;
	//cTable * Tables;
	int intQueueCnt;
	cQueue ** Queues;

	 
	/*
	void AddTable();
	void RemoveTable();
	void LockTable();
	void ReleaseTable();
	*/

	bool AddQueue(char * name, int size);
	void RemoveQueue(char * name);
	void LockQueue(char * name);
	void ReleaseQueue(char * name);
	
public:
	//void ExecuteCommand();
	bool PostMess(char * destination, char * from, char * data, int length, int ServiceNumber);

	//void PostMessage(int index, char * data, int lenght);
	
	int GetMess(char * name, char * from, char * data, int& length, int& ServiceNumber);

	//void GetMess(int index, char * data, int& length);
	
	//int GetMessagingIndex(char * clientname);


	cDataEngine();
	~cDataEngine();
	
};


/*####################################################################################*/
/*                                                                                    */
/*    %%%%  %%%%   %%%%%%%    %%%%%%%%%  %%%%%%%%%  %%%%%%%%%                         */
/*    %%%%  %%%%   %%%%%%%%   %%%   %%%  %%%%%%%%%  %%%%%%%%%                         */
/*   %%%%%%%%     %%%%  %%%  %%%%%%%%%     %%%        %%%%%                           */
/*   %%%%%%%%     %%%%%%%%   %%%%%%%%%     %%%      %%%%                              */
/*  %%%%  %%%%   %%%% %%%   %%%   %%%     %%%     %%%%%%%%%                           */
/*  %%%%  %%%%   %%%  %%%%  %%%   %%%     %%%     %%%%%%%%%                           */
/*                                                                                    */
/*      #######   ####    ####    ########       ****     ***     ***    ******       */
/*     #########  #####   ####   ##########     ******   *****   *****   *******      */
/*    ####        ######  ####  ####    ###          *  **   ** **   **       **      */
/*    ####        ######  ####  ####               **   **   ** **   **      **       */
/*     #######    ####### ####  ####              **    **   ** **   **     **        */
/*      #######   ############  ####            ******   *****   *****     **         */
/*          ####  #### #######  ####            ******    ***     ***     **          */
/*          ####  ####  ######  ####    ###                                           */
/*    #########   ####   #####   ##########      -- ---  !  ! !    +--+               */
/*     #######    ####    ####    ########      |   ",   +--! !--+ |  |               */
/*                                               -- --"     ! !__! +--+               */
/*####################################################################################*/





	
	
	
	
	
	