/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
///		CS460 - Christopher Kratz - Spring 2007                      ////
///                                                                  ////
///     Multi-Process Application Server                             ////
///                                                                  ////
///     Purpose:                                                     ////
///			This program serves as a central mechanism for processes////
///		in a multi-process application to communicate with each other////
///		and act as a single system.  This file contains the function ////
///     definitions for main system functions.                       ////
///                                                                  ////
///		Modification Log:                                            ////
///			3-12-2007	C. Kratz	Created                          ////
///                                                                  ////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

#include<afx.h>
#include<winbase.h>
#include"DataServer.h"
#include"MPS.h"




bool WaitForCommand(char * buffer, unsigned long& bytes, HANDLE pipe)
{

	/* This fuction will wait for a command to come in on the pipe, and
	when it does. puts it in the buffer */

	if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
	{
		//Read Failed
		if(GetLastError()==ERROR_BROKEN_PIPE)
		{
			//Pipe Broken
			/* TODO: Add code to handle and try to recover from broken pipe
					 this may include but is not limited to, attempting to reopen the 
					 pipe, or restarting worker process. 
					 Note: This should be part of the system standard so the client knows
							how to recover from a broken pipe as well.
			*/
			return false;
		}
		
	}

	return true;
	//cerr << "Read info.\n";
}

bool ProcessCommand(cDataEngine * data,				/* Pointer to the data class                    */
					char * clientname,				/* Name of client thread executing the command  */
					char * buffer,					/* buffer containing the command                */
					unsigned long bytes,			/* length of buffer data                        */
					char * response,				/* response to send back to client              */
					unsigned long& rbytes)			/* length of response                           */
{
	

	if(bytes <  1)
	{	
		/* Need at least one byte to be a command */
		return false;
	}

	//Interpret Command Here
	switch (buffer[0])
	{
	case 0x00:

		///Send Message
		// data format
		//  Ox00 [DESTINATION NAME] Ox00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]

		PostMessageToQueue(data, clientname, buffer, bytes, response, rbytes);
		
		break;

	case 0x01:
		//GET Message
		// response data format
		// [RETURN CODE x2 BYTES] [REMAINING MESSAGES x2 BYTES] [SERVICE CODE x2 BYTES] [FROM] 0x00 [MESSAGE DATA]
		
		GetMessageFromQueue(data, clientname, buffer, bytes, response, rbytes);
		
		break;
	case 0x02:
		///Send MultiCast Message Message
		// data format
		//  Ox02 [NUMBER OF DESTINATIONS x2 BYTES] [DESTINATION 1] Ox00 [DESTINATION 2] 0x00 .... [DESTNATION N] 0x00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]
		
		PostMulticastMessageToQueue(data, clientname, buffer, bytes, response, rbytes);


		break;


	default:
		//unrecognized
		return false;
		break;
	}
	


	return true;

}


bool SendResponse(char * response,			/* The response to place on the pipe */	
				  unsigned long rbytes,		/* The length of the response        */
				  HANDLE pipe)				/* Handle to the pipe                */
{
	unsigned long bytes;

	/* Write the response to the pipe */		
	if(WriteFile(pipe, response, rbytes, &bytes, NULL) == 0)
	{
		/* Add error handling for busted pipes */
		//WriteFailed
		return false;
	}
	return true;
}

void PostMessageToQueue(cDataEngine * data,				/* Pointer to the data class                    */
							char * clientname,				/* Name of client thread executing the command  */
							char * buffer,					/* buffer containing the command                */
							unsigned long bytes,			/* length of buffer data                        */
							char * response,				/* response to send back to client              */
							unsigned long& rbytes)			/* length of response                           */
{
	///Send Message
	// data format
	//  Ox00 [DESTINATION NAME] Ox00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]
	
	int ServiceCode = 0;
	char destname[SIZE_AGENTNAME];

	/* Grab destination name, starts at &buffer[1] and is null terminated */
	strncpy(destname, &buffer[1], bytes);

	/* Get Service Code From Message */
	ServiceCode = (int)buffer[strlen(destname) + 2] * 256;
	ServiceCode += (int)buffer[strlen(destname) + 3];
	
	/* Calculate start of message and message length */
	int messstart = strlen(destname) + 4;
	int messlen = bytes-messstart;

	/* Post message to destinations queue */
	if(!data->PostMess(destname, clientname, &buffer[messstart], messlen, ServiceCode))
	{
		/*Could not post message*/
		response[0] = 0x01;
		rbytes = 1;
	}
	else
	{
		//Generate response
		response[0] = 0x00;
		rbytes=1;
	}

	return;
}

void GetMessageFromQueue(cDataEngine * data,				/* Pointer to the data class                    */
							char * clientname,				/* Name of client thread executing the command  */
							char * buffer,					/* buffer containing the command                */
							unsigned long bytes,			/* length of buffer data                        */
							char * response,				/* response to send back to client              */
							unsigned long& rbytes)			/* length of response                           */
{
	//GET Message
	// response data format
	// [RETURN CODE x2 BYTES] [REMAINING MESSAGES x2 BYTES] [SERVICE CODE x2 BYTES] [FROM] 0x00 [MESSAGE DATA]

	int ServiceCode;
	char get_message[SIZE_MESSAGE];
	int get_message_len;
	int leftinQ;
	char from[SIZE_AGENTNAME];
	
	/* Retrieve the next message from the queue */
	leftinQ = data->GetMess(clientname, from, get_message, get_message_len, ServiceCode);

	if(leftinQ >= 0)
	{
		response[0]=0;						/* Response Code For Success is 0 */
		response[1]=0;						/* Return as 16-bit integer       */

		response[2]= (leftinQ / 256);		/* Return Number of messages left */
		response[3]= (leftinQ % 256);		/* Return as 16-bit integer       */

		response[4]= (ServiceCode / 256);	/* Return ServiceCode of message  */
		response[5]= (ServiceCode % 256);	/* Return as 16-bit integer		  */

		int len = strlen(from);

		/* Copy from into the response*/
		int i;
		for(i=0;i<len; i++)
		{
			response[i+6] = from[i];
		}

		/* Terminate From */
		response[i+6] = 0x00;
		int offset=len+1+6;

		/* Copy Data into response */
		for(i=0;i<get_message_len; i++)
		{
			response[i+offset] = get_message[i];
		}
		rbytes=get_message_len+offset;

	}
	else
	{
		/* NO MESSAGES TO RETIRIEVE */
		/* Format: 0x00             */

		response[0] = 0x01;
		rbytes=1;
	}

}



void PostMulticastMessageToQueue(cDataEngine * data,				/* Pointer to the data class                    */
									char * clientname,				/* Name of client thread executing the command  */
									char * buffer,					/* buffer containing the command                */
									unsigned long bytes,			/* length of buffer data                        */
									char * response,				/* response to send back to client              */
									unsigned long& rbytes)			/* length of response                           */
{
	int cntDest;
	char (*Dests)[SIZE_AGENTNAME];
	char destname[SIZE_AGENTNAME];

	/* Get Number of destinations*/
	cntDest = (int)buffer[1] * 256;
	cntDest += (int)buffer[2];

	/* Destination Name array */
	Dests = new char[cntDest][SIZE_AGENTNAME];
	if(Dests == NULL)
	{
		/*FAILED TO GET MEMORY*/
		response[0] = 0x01;
		rbytes=1;
		return;
	}

	int i;
	int offset = 3;
	for(i=0;i<cntDest;i++)
	{
		/* Get destination name and increment offset */
		strncpy(Dests[i], &buffer[offset], SIZE_AGENTNAME);
		offset += strlen(Dests[i]) + 1;
	}

	int ServiceCode;
	/* Get Service Code From Message */
	ServiceCode = (int)buffer[strlen(destname) + offset] * 256;
	ServiceCode += (int)buffer[strlen(destname) + offset+1];
	offset += 2;

	/* Calculate start of message and message length */
	int messstart = offset;
	int messlen = bytes-messstart;
	
	int sent=0;
	for(i=0;i<cntDest;i++)
	{
		
		/* Post message to destinations queue */
		if(!data->PostMess(Dests[i], clientname, &buffer[messstart], messlen, ServiceCode))
		{
			/*Could not post message*/
		}
		else
		{
			/*Message Sent*/
			sent++;
		}
	}

	if(sent == 0)
	{
		response[0] = 0x02;
		rbytes  = 1;
	}
	else
	{
		response[0] = 0x00;
		response[1] = sent / 256;
		response[2] = sent % 256;
		rbytes=3;
	}
	return;
}




/*####################################################################################*/
/*                                                                                    */
/*    %%%%  %%%%   %%%%%%%    %%%%%%%%%  %%%%%%%%%  %%%%%%%%%                         */
/*    %%%%  %%%%   %%%%%%%%   %%%   %%%  %%%%%%%%%  %%%%%%%%%                         */
/*   %%%%%%%%     %%%%  %%%  %%%%%%%%%     %%%        %%%%%                           */
/*   %%%%%%%%     %%%%%%%%   %%%%%%%%%     %%%      %%%%                              */
/*  %%%%  %%%%   %%%% %%%   %%%   %%%     %%%     %%%%%%%%%                           */
/*  %%%%  %%%%   %%%  %%%%  %%%   %%%     %%%     %%%%%%%%%                           */
/*                                                                                    */
/*      #######   ####    ####    ########       ****     ***     ***    ******       */
/*     #########  #####   ####   ##########     ******   *****   *****   *******      */
/*    ####        ######  ####  ####    ###          *  **   ** **   **       **      */
/*    ####        ######  ####  ####               **   **   ** **   **      **       */
/*     #######    ####### ####  ####              **    **   ** **   **     **        */
/*      #######   ############  ####            ******   *****   *****     **         */
/*          ####  #### #######  ####            ******    ***     ***     **          */
/*          ####  ####  ######  ####    ###                                           */
/*    #########   ####   #####   ##########      -- ---  !  ! !    +--+               */
/*     #######    ####    ####    ########      |   ",   +--! !--+ |  |               */
/*                                               -- --"     ! !__! +--+               */
/*####################################################################################*/

