/* Track class implementation */
#include<stdio.h>
#include "track.h"


Track::Track()
{
	SectionCount = 0;
	NodeCount = 0;
	SensorCount = 0;
}

Track::~Track()
{
}

unsigned long Track::GetNodeCount()
{
	return NodeCount;
}

unsigned long Track::GetSectionCount()
{
	return SectionCount;
}

bool Track::AddNode(unsigned long id, unsigned long to, unsigned long switchgroup, unsigned long sensor)
{

	if(NodeCount >= MAX_NODES)
	{
		return false;
	}

	nodes[NodeCount].ID = id;
	nodes[NodeCount].OwnerID = 0xffffffff;
	nodes[NodeCount].To = to;
	nodes[NodeCount].SwitchGroup = switchgroup;
	nodes[NodeCount].ActiveSensorID = sensor;
	NodeCount++;

	return true;
}

bool Track::AddSection(unsigned long	id,
					   double			ax,
					   double			ay,
					   double			bx,
					   double			by,
					   double			radius,
					   unsigned long	nodeA,
					   unsigned long	nodeB)
{

	if(SectionCount >= MAX_SECTIONS)
	{
		return false;
	}

	sections[SectionCount].ID = id;
	sections[SectionCount].Ax = ax;
	sections[SectionCount].Ay = ay;
	sections[SectionCount].Bx = bx;
	sections[SectionCount].By = by;
	sections[SectionCount].Radius = radius;
	sections[SectionCount].NodeA = nodeA;
	sections[SectionCount].NodeB = nodeB;
	sections[SectionCount].OwnerID = 0xffffffff;
	SectionCount++;

	return true;

}

bool Track::AddSensor(unsigned long ID, unsigned long aiu, unsigned long pin, unsigned long sec, double pos)
{
	if(SensorCount >= MAX_SENSORS)
	{
		return false;
	}

	sensors[SensorCount].ID = ID;
	sensors[SensorCount].AIU = aiu;
	sensors[SensorCount].pin  = pin;
	sensors[SensorCount].pos  = pos;
	sensors[SensorCount].section  = sec;
	sensors[SensorCount].value   = false;
	sensors[SensorCount].lastvalue   = false;

	SensorCount++;

	return true;
}


bool Track::LoadConfigFromFile(char * filename)
{
	Clear();

	FILE * file = fopen(filename, "rt");
	if(file == NULL)
	{
		return false;
	}

	char buffer[SIZE_TRACKCONFIG];

	while(fgets(buffer, SIZE_TRACKCONFIG, file) != NULL)
	{
		unsigned int id, nodea, nodeb, to;
		float ax,ay, bx, by, r;
		unsigned int sid, aiu, pin, sec;
		float pos; 
		if(sscanf(buffer, "SECTION ID=%u AX=%f AY=%f BX=%f BY=%f R=%f NODEA=%u NODEB=%u",
							&id, &ax, &ay, &bx, &by, &r, &nodea, &nodeb) == 8)
		{
			AddSection(id, ax, ay, bx, by, r, nodea, nodeb);
		}
		else if(sscanf(buffer, "NODE ID=%u TO=%u", &id, &to) == 2)
		{
			AddNode(id, to);
		}
		else if(sscanf(buffer, "SENSOR ID=%u AIU=%u PIN=%u SECTION=%u POS=%f",
							&sid, &aiu, &pin, &sec, &pos) == 5)
		{
			AddSensor(sid, aiu, pin, sec, pos);
		}
		else
		{
			/*invalid line in file, skipping*/
		}
	}

	fclose(file);

	return true;
	
}

unsigned long Track::GetSensorCount()
{
	return SensorCount;
}

Section Track::GetSectionByIndex(unsigned long index)
{
	return sections[index];
}

Node Track::GetNodeByIndex(unsigned long index)
{
	return nodes[index];
}

Sensor Track::GetSensorByIndex(unsigned long index)
{
	return sensors[index];
}


Section Track::GetSectionById(unsigned long id)
{
	unsigned long i;
	for(i=0;i<SectionCount;i++)
	{
		if(sections[i].ID == id)
		{
			return sections[i];
		}
	}
	/* not found */
	Section ret;
	ret.ID = 0;
	return ret;
}

Node Track::GetNodeById(unsigned long id) /* don't use, node id's are not be unique */
{
	/*
	unsigned long i;
	for(i=0;i<NodeCount;i++)
	{
		if(nodes[i].ID == id)
		{
			return nodes[id];
		}
	}
	*/
	/* not found */
	Node ret;
	ret.ID = 0;
	return ret;
}


Node Track::GetActiveNode(unsigned long id)
{
	unsigned long i;
	for(i=0;i<NodeCount;i++)
	{
		if(nodes[i].ID == id && nodes[i].Active )
		{
			return nodes[i];
		}
	}
	/* not found */
	Node ret;
	ret.ID = 0;
	return ret;
}

bool Track::ToggleNode(unsigned long ID)
{
	unsigned long i;
	for(i=0;i<NodeCount;i++)
	{
		if(nodes[i].ID == ID)
		{
			nodes[i].Active = !nodes[i].Active;
		}
	}

	return true;
}

Sensor Track::GetSensorById(unsigned long id)
{
	unsigned long i;
	for(i=0;i<SensorCount;i++)
	{
		if(sensors[i].ID == id)
		{
			return sensors[i];
		}
	}
	/* not found */
	Sensor ret;
	ret.ID = 0;
	return ret;
}


void Track::Clear()
{
	SectionCount = 0;
	NodeCount = 0;
	SensorCount = 0;
}

bool Track::AddSection(Section s)
{
	
	if(SectionCount >= MAX_SECTIONS)
	{
		return false;
	}

	sections[SectionCount] = s;

	SectionCount++;

	return true;
}

bool Track::AddNode(Node n)
{
	
	if(NodeCount >= MAX_NODES)
	{
		return false;
	}

	nodes[NodeCount] = n;

	NodeCount++;

	return true;
}

bool Track::AddSensor(Sensor s)
{
	
	if(SensorCount >= MAX_SENSORS)
	{
		return false;
	}

	sensors[SensorCount] = s;

	SensorCount++;

	return true;
}

bool Track::SetSensorValue(unsigned long index, bool val)
{
	sensors[index].lastvalue = sensors[index].value;
	sensors[index].value = val;
	if(sensors[index].lastvalue != val)
		return true;
	else
		return false;

}

bool Track::SetNodeOwner(unsigned long ID, unsigned long owner)
{
		unsigned long i;
		bool found = false;
	for(i=0;i<NodeCount;i++)
	{
		if(nodes[i].ID)
		{
			nodes[i].OwnerID = owner;
			found = true;
		}
	}
	return found;
}

bool Track::SetSectionOwner(unsigned long ID, unsigned long owner)
{
		unsigned long i;
	for(i=0;i<SectionCount;i++)
	{
		if(sections[i].ID == ID)
		{
			sections[i].OwnerID = owner;
			return true;
		}
	}
	/* not found */
	return false;
}

bool Track::ClearNodeOwner(unsigned long ID)
{
		unsigned long i;
		bool found = false;
	for(i=0;i<NodeCount;i++)
	{
		if(nodes[i].ID)
		{
			nodes[i].OwnerID = 0xffffffff;
			found = true;
		}
	}
	return found;
}

bool Track::ClearSectionOwner(unsigned long ID)
{
			unsigned long i;
	for(i=0;i<SectionCount;i++)
	{
		if(sections[i].ID == ID)
		{
			sections[i].OwnerID = 0xffffffff;
			return true;
		}
	}
	/* not found */
	return false;
}