/* Track data types */

const MAX_SECTIONS	= 255;
const MAX_NODES		= 255;
const MAX_SENSORS	= 255;
const SIZE_TRACKCONFIG = 255;


struct Section
{
	unsigned long ID;
	unsigned long OwnerID;

	double Ax;
	double Ay;
	
	double Bx;
	double By;

	unsigned long NodeA;
	unsigned long NodeB;

	double Radius;
	
	bool Active;

};

struct Node
{
	unsigned long ID;
	unsigned long OwnerID;

	unsigned long To;
	
	unsigned long SwitchGroup; /* ID of switch-it or 0 */
	unsigned long ActiveSensorID; /* ID of sensor tied to switch */

	bool Active;
};

struct Sensor
{
	unsigned long ID;
	unsigned long AIU;
	unsigned long pin;

	unsigned long section;

	bool value;
	bool lastvalue;

	double pos;
};


class Track
{
private:
	unsigned long SectionCount;
	unsigned long NodeCount;
	unsigned long SensorCount;

	Section sections[MAX_SECTIONS];
	Node nodes[MAX_NODES];
	Sensor sensors[MAX_SENSORS];

	


	PostDataToServer();

public:
	
	bool LoadConfigFromFile(char * filename);

	Section GetSectionById(unsigned long ID);
	Node GetNodeById(unsigned long ID);
	Sensor GetSensorById(unsigned long ID);
	Node GetActiveNode(unsigned long ID);

	bool ToggleNode(unsigned long ID);


	Section GetSectionByIndex(unsigned long index);
	Node GetNodeByIndex(unsigned long index);
	Sensor GetSensorByIndex(unsigned long index);
	
	bool SetSectionOwner(unsigned long ID, unsigned long owner);
	bool SetNodeOwner(unsigned long ID, unsigned long owner);
	bool ClearSectionOwner(unsigned long ID);
	bool ClearNodeOwner(unsigned long ID);

	unsigned long GetSectionCount();
	unsigned long GetNodeCount();
	unsigned long GetSensorCount();

	bool SetSensorValue(unsigned long index, bool val);

	void Clear();

		
	bool AddSection(unsigned long	id,
					double			ax,
					double			ay,
					double			bx,
					double			by,
					double			radius,
					unsigned long	nodeA,
					unsigned long	nodeB);

	bool AddNode(	unsigned long	id,
					unsigned long	to,
					unsigned long   switchgroup = 0,
					unsigned long   activesensor= 0);

	bool AddSensor( unsigned long id,
					unsigned long AIU,
					unsigned long pin,
					unsigned long sec,
					double pos);

	bool AddSection(Section s);
	bool AddNode(Node n);
	bool AddSensor(Sensor s);


	Track();
	~Track();

};

struct Train
{
	unsigned long id;
	long speed;			/*Speed is in percent of full speed 0 - 100*/
	long targetspeed;
	char slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
};

struct Resource
{
	char type;
	unsigned long id;
	unsigned long owner;
	bool active;
};
	
	
	
	
	
	
	
