/***************************************************/
/* API CALLS SPECIFIC TO THE TOS SYSTEM            */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*  CHRISTOPHER KRATZ SNC 4/4/2007                 */
/***************************************************/
/* TOS_CLIENT.CPP */

#include<windows.h>
#include"track.h"
#include"kmp_client_api.h"
#include"tos_api.h"

/* TRACK MANAGER RELATED FUNCTIONS */

bool TOS_RequestTrackData(HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRACK_DATA, TRACK_MAN, pipe);
}

bool TOS_RequestTrackDataUpdates(HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRACK_DATA_UPDATES, TRACK_MAN, pipe);
}
	
bool TOS_RequestLoadTrackFromFile(char * filename, HANDLE pipe)
{
	/* Message Format */
	/* [Filename]0x00 */
	if(filename == NULL)
		return false;
		
	int len = strlen(filename);
	
	return SendMessageToPeer(filename, len, SC_LOAD_TRACK_FROM_FILE, TRACK_MAN, pipe);
}
	
bool TOS_SendTrackData(Track * track, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [# of sections (x4)][#of nodes (x4)][Sections].... [Nodes]....  */
	if(track==NULL)
		return false;
		
	if(destination == NULL)
		return false;
		
	char * message = new char[track->GetSectionCount() * sizeof(Section) + track->GetNodeCount() * sizeof(Node) + 16];
	
	if(message == NULL)
		return false;
		
	unsigned long offset = 0;
	
	message[0] = (char)(track->GetSectionCount() / 0x01ffffff);
	message[1] = (char)(track->GetSectionCount() / 0x0001ffff);
	message[2] = (char)(track->GetSectionCount() / 0x000001ff);
	message[3] = (char)(track->GetSectionCount() % 0x000001ff);
	
	message[4] = (char)(track->GetNodeCount() / 0x01ffffff);
	message[5] = (char)(track->GetNodeCount() / 0x0001ffff);
	message[6] = (char)(track->GetNodeCount() / 0x000001ff);
	message[7] = (char)(track->GetNodeCount() % 0x000001ff);
	
	offset=8;
	
	unsigned long i;
	for(i=0;i<track->GetSectionCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Section);j++)
		{
			Section sec = track->GetSectionByIndex(i);
			message[offset+j] = ((char *)(&sec))[j];
		}
		offset += sizeof(Section);
	}
	
	for(i=0;i<track->GetNodeCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Node);j++)
		{
			Node node = track->GetNodeByIndex(i);
			message[offset+j] = ((char *)(&node))[j];
		}
		offset += sizeof(Node);
	}
		
	bool ret = SendMessageToPeer(message, offset, SC_TRACK_DATA, destination, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_SendTrackData(Track * track, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [# of sections (x4)][#of nodes (x4)][Sections].... [Nodes]....  */
	if(track==NULL)
		return false;
		
	if(destination == NULL)
		return false;
		
	char * message = new char[track->GetSectionCount() * sizeof(Section) + track->GetNodeCount() * sizeof(Node) + 16];
	
	if(message == NULL)
		return false;
		
	unsigned long offset = 0;
	
	message[0] = (char)(track->GetSectionCount() / 0x01ffffff);
	message[1] = (char)(track->GetSectionCount() / 0x0001ffff);
	message[2] = (char)(track->GetSectionCount() / 0x000001ff);
	message[3] = (char)(track->GetSectionCount() % 0x000001ff);
	
	message[4] = (char)(track->GetNodeCount() / 0x01ffffff);
	message[5] = (char)(track->GetNodeCount() / 0x0001ffff);
	message[6] = (char)(track->GetNodeCount() / 0x000001ff);
	message[7] = (char)(track->GetNodeCount() % 0x000001ff);
	
	offset=8;
	
	unsigned long i;
	for(i=0;i<track->GetSectionCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Section);j++)
		{
			Section sec = track->GetSectionByIndex(i);
			message[offset+j] = ((char *)(&sec))[j];
		}
		offset += sizeof(Section);
	}
	
	for(i=0;i<track->GetNodeCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Node);j++)
		{
			Node node = track->GetNodeByIndex(i);
			message[offset+j] = ((char *)(&node))[j];
		}
		offset += sizeof(Node);
	}
		
	bool ret = SendMulticastMessageToPeer(message, offset, SC_TRACK_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}

/* TRAIN MANAGER */
bool TOS_SetTrainSpeed(unsigned long TrainID, unsigned long Speed, bool direction, HANDLE pipe)
{
	/* Message Format */
	/* [Train ID (x2)][Speed (x2)][Direction (x1)] */
	
	char message[5];
	
	message[0] = (unsigned char)(TrainID / 256);
	message[1] = (unsigned char)(TrainID % 256);
	message[2] = (unsigned char)(Speed / 256);
	message[3] = (unsigned char)(Speed % 256);
	if(direction)
	{
		message[4] = 1;
	}
	else
	{
		message[4] = 0;
	}
	
	return SendMessageToPeer(message, 5, SC_SET_TRAIN_SPEED, TRAIN_MAN, pipe);
}
	
bool TOS_SetTrainAccessory(unsigned long TrainID, unsigned long Acc, unsigned long state, HANDLE pipe)
{
	/* Message Format */
	/* FUNCTION SKIPPED */
	return false;
}

bool TOS_RequestTrainUpdates(unsigned long interval, HANDLE pipe)
{
	/* Message Format */
	/* [interval in ms (x2)] */
	
	char message[2];
	
	message[0] = (unsigned char)(interval / 256);
	message[1] = (unsigned char)(interval % 256);
	
	return SendMessageToPeer(message, 2, SC_REQUEST_TRAIN_UPDATES, TRAIN_MAN, pipe);
}
	
bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [Train Count (x2)][TrainList [Id (x2)][Speed(x2)][Direction(x1)][Slope(x1)]
							[Section(x2)][Section Pos(x8)][X (x8)][Y (x8)] .....   ] */   
	
	/*
	unsigned long id;
	unsigned long speed;
	bool direction;
	bool slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
	*/

	char * message = new char[2 + (34 * traincnt)];
	
	if(message == NULL)
		return false;
	
	unsigned long offset = 0;
	unsigned long i;
	
	message[0] = (unsigned char)(traincnt / 256);
	message[1] = (unsigned char)(traincnt % 256);
	offset = 2;
	for(i=0; i< traincnt; i++)
	{
		message[offset] = (unsigned char)(trainlist[i].id / 256);
		message[offset+1] = (unsigned char)(trainlist[i].id % 256);
		message[offset+2] = (unsigned char)(trainlist[i].speed / 256);
		message[offset+3] = (unsigned char)(trainlist[i].speed % 256);
		if(trainlist[i].direction)
			message[offset+4] = 1;
		else
			message[offset+4] = 0;
			
		if(trainlist[i].slope)
			message[offset+5] = 1;
		else
			message[offset+5] = 0;
			
		message[offset+6] = (unsigned char)(trainlist[i].section / 256);
		message[offset+7] = (unsigned char)(trainlist[i].section % 256);
		offset += 8;
		unsigned long j;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].sectionpos))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].x))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].y))[j];
		}
		offset += 8;
	}

	bool ret = SendMessageToPeer(message, offset, SC_TRAIN_DATA, destination, pipe);
	delete[] message;
	return ret;
	
}

bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [Train Count (x2)][TrainList [Id (x2)][Speed(x2)][Direction(x1)][Slope(x1)]
							[Section(x2)][Section Pos(x8)][X (x8)][Y (x8)] .....   ] */   

	/*
	unsigned long id;
	unsigned long speed;
	bool direction;
	bool slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
	*/

	char * message = new char[2 + (34 * traincnt)];
	
	if(message == NULL)
		return false;
	
	unsigned long offset = 0;
	unsigned long i;
	
	message[0] = (unsigned char)(traincnt / 256);
	message[1] = (unsigned char)(traincnt % 256);
	offset = 2;
	for(i=0; i< traincnt; i++)
	{
		message[offset] = (unsigned char)(trainlist[i].id / 256);
		message[offset+1] = (unsigned char)(trainlist[i].id % 256);
		message[offset+2] = (unsigned char)(trainlist[i].speed / 256);
		message[offset+3] = (unsigned char)(trainlist[i].speed % 256);
		if(trainlist[i].direction)
			message[offset+4] = 1;
		else
			message[offset+4] = 0;
			
		if(trainlist[i].slope)
			message[offset+5] = 1;
		else
			message[offset+5] = 0;
			
		message[offset+6] = (unsigned char)(trainlist[i].section / 256);
		message[offset+7] = (unsigned char)(trainlist[i].section % 256);
		offset += 8;
		unsigned long j;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].sectionpos))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].x))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].y))[j];
		}
		offset += 8;
	}

	bool ret = SendMulticastMessageToPeer(message, offset, SC_TRACK_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_RequestTrainData(HANDLE pipe)
{

	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRAIN_DATA, TRAIN_MAN, pipe);

}


bool TOS_AddTrain(unsigned long TrainID, unsigned long SectionID, double SectionPos, HANDLE pipe)
{
	/* MessageID */
	/* [TrainID (x2)][SectionID(x2)][SectionPos(x8)] */
	
	char message[14];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	message[2] = SectionID / 256;
	message[3] = SectionID % 256;
	
	unsigned long i;
	for(i=0; i<8; i++)
	{
		message[4+i] = ((char*)(&SectionPos))[i];
	}
	
	return SendMessageToPeer(message, 12, SC_ADD_TRAIN, TRAIN_MAN, pipe);
	
}
	
bool TOS_RemoveTrain(unsigned long TrainID, HANDLE pipe)
{
	/* MessageID */
	/* [TrainID (x2)] */
	
	char message[3];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	return SendMessageToPeer(message, 2, SC_REMOVE_TRAIN, TRAIN_MAN, pipe);
	
}


/* RESOURCE MANAGER */
bool TOS_RequestOwnership(unsigned long TrainID, unsigned char type, unsigned long ResourceID, HANDLE pipe)
{
	/* Message Format */
	/* [TrainID(x2)][Type(x1)][ResourceID(x2)] */
	
	char message[5];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	message[2] = type;
	
	message[3] = ResourceID / 256;
	message[4] = ResourceID % 256;
	
	return SendMessageToPeer(message, 5, SC_REQUEST_OWNERSHIP, RESOURCE_MAN, pipe);
	
}

bool TOS_ActivateNode(unsigned long NodeID, HANDLE pipe)
{
	/* Message Format */
	/* [NodeId(x2)] */
	
	char message[2];
	
	message[0] = NodeID / 256;
	message[1] = NodeID % 256;
	
	return SendMessageToPeer(message, 2, SC_ACTIVATE_NODE, RESOURCE_MAN, pipe);
	
}
	
bool TOS_RequestResourceUpdates(unsigned long Interval, HANDLE pipe)
{
	/* Message Format */
	/* [Interval(x2)] */
	
	char message[2];
	
	message[0] = Interval / 256;
	message[1] = Interval % 256;
	
	return SendMessageToPeer(message, 2, SC_REQUEST_RESOURCE_UPDATES, RESOURCE_MAN, pipe);
	
}
	
bool TOS_GrantResource(unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [Type (x1)][ResourceID(x2)] */
	
	char message[3];
		
	message[0] = type;
	message[1] = ResourceID / 256;
	message[2] = ResourceID % 256;
	
	return SendMessageToPeer(message, 3, SC_RESOURCE_GRANT, destination, pipe);
	
}
	
bool TOS_DenyResource(unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [Type (x1)][ResourceID(x2)] */
	
	char message[3];
		
	message[0] = type;
	message[1] = ResourceID / 256;
	message[2] = ResourceID % 256;
	
	return SendMessageToPeer(message, 3, SC_RESOURCE_DENY, destination, pipe);
	
}

bool TOS_SendResourceData(unsigned long cnt, Resource * res, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [cnt (x2)][Resource[Type(x1)][ID(x2)][Owner(x2)][Active(x1)]] ........  */
	
	char * message = new char[2+(cnt*6)];
	
	if(message==NULL)
	{ 
		return false;
	}
	
	message[0] = cnt / 256;
	message[1] = cnt % 256;
	unsigned long offset = 2;
	unsigned long i;
	for(i=0;i<cnt;i++)
	{
		message[offset] 	= res[i].type;
		message[offset+1] 	= res[i].id / 256;
		message[offset+2] 	= res[i].id % 256;
		message[offset+3]	= res[i].owner / 256;
		message[offset+4]	= res[i].owner % 256;
		message[offset+5]	= res[i].active;
		offset += 6;
	}
	
	bool ret = SendMessageToPeer(message, offset, SC_RESOURCE_DATA, destination, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_SendResourceData(unsigned long cnt, Resource * res, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [cnt (x2)][Resource[Type(x1)][ID(x2)][Owner(x2)][Active(x1)]] ........  */
	
	char * message = new char[2+(cnt*6)];
	
	if(message==NULL)
	{ 
		return false;
	}
	
	message[0] = cnt / 256;
	message[1] = cnt % 256;
	unsigned long offset = 2;
	unsigned long i;
	for(i=0;i<cnt;i++)
	{
		message[offset] 	= res[i].type;
		message[offset+1] 	= res[i].id / 256;
		message[offset+2] 	= res[i].id % 256;
		message[offset+3]	= res[i].owner / 256;
		message[offset+4]	= res[i].owner % 256;
		message[offset+5]	= res[i].active;
		offset += 6;
	}
	
	bool ret = SendMulticastMessageToPeer(message, offset, SC_RESOURCE_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}

/* DCC MANAGER */
bool TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe)
{
	/* message format */
	/* [DCC address (x2)][Speed(x2)][direction(x1)] */
	char message[6];
	
	message[0] = TrainAddress / 256;
	message[1] = TrainAddress % 256;
	message[2] = Speed / 256;
	message[3] = Speed % 256;
	message[4] = direction;
	
	return SendMessageToPeer(message, 5, SC_SET_TRAIN_SPEED_DCC, DCC_MAN, pipe);
}
	
bool TOS_SetTrainAccessoryDCC(unsigned long TrainAddress, unsigned long Accessory, unsigned long value, HANDLE pipe)
{
	/* message format */
	/* [dcc address(x2)][accessroy(x2)][value(x1)] */
	
	char message[6];
	
	message[0] = TrainAddress / 256;
	message[1] = TrainAddress % 256;
	message[2] = Accessory / 256;
	message[3] = Accessory % 256;
	message[4] = value;
	
	return SendMessageToPeer(message, 5, SC_SET_TRAIN_ACCESSORY_DCC, DCC_MAN, pipe);
}
	
bool TOS_SetTurnout(unsigned long Address, char value, HANDLE pipe)
{
	/* message format */
	/* [dcc address(x2)][value(x1)] */
	
	char message[4];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = value;
	
	return SendMessageToPeer(message, 3, SC_SET_TURNOUT, DCC_MAN, pipe);
}

bool TOS_ReadAIU(unsigned long Address, unsigned long Input, HANDLE pipe)
{
	/*message format */
	/* [address(x2)][input(x1)] */
	
	char message[4];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = Input;
	
	return SendMessageToPeer(message, 3, SC_READ_AIU, DCC_MAN, pipe);
}

bool TOS_SendAIUUpdate(unsigned long Address, unsigned long input, unsigned long value, char * destination, HANDLE pipe)
{
	/* message format */
	/* [address(x2)][input(x1)][value(x1)] */
	
	char message[5];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = input;
	message[3] = value;
	
	return SendMessageToPeer(message, 4, SC_AIU_UPDATES, destination, pipe);
}

bool TOS_SendAIUUpdate(unsigned long Address, unsigned long input, unsigned long value, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* message format */
	/* [address(x2)][input(x1)][value(x1)] */
	
	char message[5];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = input;
	message[3] = value;
	
	return SendMulticastMessageToPeer(message, 4, SC_AIU_UPDATES, destination, destcnt, pipe);
}

bool TOS_SendDCCPacket(unsigned long length, char * data, HANDLE pipe)
{
	/*message format */
	/* [length(x2)][data(x?)] */
	
	char * message = new char[length];
	if(message==NULL)
		return false;
		
	unsigned long i;
	for(i=0;i<length;i++)
	{
		message[i] = data[i];
	}
	
	bool ret = SendMessageToPeer(message, length, SC_SEND_DCC_PACKET, DCC_MAN, pipe);
	delete[] message;
	
	return ret;
}

bool TOS_DCCAck(char * destination, HANDLE pipe)
{
	return false;
}

bool TOS_DCCNack(unsigned long error, char * destination, HANDLE pipe)
{
	return false;
}






















