/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
///		CS460 - Christopher Kratz - Spring 2007                      ////
///                                                                  ////
///     Multi-Process Application Server                             ////
///                                                                  ////
///     Purpose:                                                     ////
///			This programm serves as a central mechanism for processes////
///		in a multi-process application to communicate with each other////
///		and act as a single system.                                  ////
///                                                                  ////
///		Modification Log:                                            ////
///			2-27-2007	C. Kratz	Created                          ////
///                                                                  ////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////


#include<afx.h>
#include<process.h>
#include<string.h>
#include<iostream.h>
#include<winbase.h>
#include"DataServer.h"		//cDataEngine class



void StartProcess(char * name,char * pipename);			//Executes a process

HANDLE StartWorkerThread(char * clientname,				//Begins the worker thread
						 char * pipename, 
						 cDataEngine * data);

unsigned int _stdcall WorkerThread(void * data);		//Worker thread


struct WorkerThreadStartInfo
{
	cDataEngine * data;		
	char ClientName[255];
	char PipeName[255];
};

void main()
{

	/*	this class holds all the functionality of the data and messaging
		system. It is put in the for of a class so it can be passed easily
		to the worker threads when they are initiated.						*/
	cDataEngine test;


	/*	Array to hold handles of started processes. Right now this is configured
		to start the processes and wait for them to end before the main thread
		terminates. This functionality could be modified later to provide a way
		to monitor, stop, or restart worker thread.							*/
	HANDLE t[2];

	t[0] = StartWorkerThread("Agent1", "\\\\.\\pipe\\Agent1", &test);
	t[1] = StartWorkerThread("Agent2", "\\\\.\\pipe\\Agent2", &test);

	/* Wont return until both threads are in signaled state					*/
	WaitForMultipleObjects(2, t, true, INFINITE);
}


void StartProcess(char * name,char * pipename)
{
	/* Starts a process using win32 api CreateProcess funciton */

	_PROCESS_INFORMATION info;		// Structure CreateProcess uses to return information on
									//		the created process

	STARTUPINFO sinfo;				// Struture containing information on how to start the process
									//		required for CreateProcess

	GetStartupInfo(&sinfo);			// Uses this processes startup info for the new process


	char command[255];				// Builds a commmand line for the new process
	strcpy(command, "name");
	strcat(command, " ");
	strcat(command, pipename);

	if(CreateProcess(name, command, NULL, NULL, FALSE, 
						CREATE_NEW_CONSOLE, NULL, NULL, &sinfo, &info)==0)
	{
		//cout << "CreateProcess Failed...\n";
		return;
	}

}

HANDLE StartWorkerThread(char * clientname, char * pipename, cDataEngine * data)
{
	/* Begins the worker thread */

	/* IMPORTANT: the WorkerThreadStartInfo structure is uses to pass data
					to the worker thread, since using the beginthreadex function
					you can only specify a single void pointer as a parameter.
					Using dynamic memory is important, if you use a local variable
					to pass the info, you cannot garuantee that the worker thread
					will get the data. Local variables reside in the current stack
					frame, but once this function returns, that area in memory will
					be replaced by the next function called in the main thread and 
					will change the data that the worker thread can see.	*/

	WorkerThreadStartInfo * startinfo = new WorkerThreadStartInfo;

	/* Copy pipe name, client name, and data pointer into startinfo */
	strcpy(startinfo->ClientName, clientname);
	strcpy(startinfo->PipeName, pipename);
	startinfo->data = data;

	/*begin thread*/
	unsigned int thread;
	return (HANDLE)_beginthreadex(NULL, 0, WorkerThread, startinfo, 0, &thread);


}


unsigned int _stdcall WorkerThread(void * clientdata)
{
	/* This is the worker thread. It is responsible for maintaining the pipe
		interpreting and acting on commands it recieves from client processes */

	char clientname[255];
	char pipename[255];

	strcpy(clientname, ((WorkerThreadStartInfo*)clientdata)->ClientName);
	strcpy(pipename, ((WorkerThreadStartInfo*)clientdata)->PipeName);
	cDataEngine * data = ((WorkerThreadStartInfo*)clientdata)->data;

	/* This was dynamically created in StartWorkerThread, must deallocate */
	delete clientdata;


	/* Create the pipe. F1 on the function name for details of arguments. */
	HANDLE pipe		=	CreateNamedPipe(pipename,
										PIPE_ACCESS_DUPLEX,
										PIPE_TYPE_MESSAGE | PIPE_READMODE_BYTE,
										PIPE_UNLIMITED_INSTANCES,
										256,
										256,
										0,
										NULL);


	/* Debug info for console testing, must change later */
	cerr << "Connecting Pipe " << clientname << "\n";

	/* Wait for the client to connect */
	if( ConnectNamedPipe(pipe, NULL) == 0 )
	{	
		DWORD result = GetLastError();
		if(result != ERROR_PIPE_CONNECTED)
		{
			return 0;
		}
	}

	cerr << "Connected Pipe " << clientname << "\n";

	//Used to store response
	char response[255];
	int resplength;

	char buffer[255];
	unsigned long bytes=0;

	while(true)
	{
		if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
		{
			//Read Failed
			if(GetLastError()==ERROR_BROKEN_PIPE)
			{
				//Pipe Broken
				return 0;
			}
			
		}

		cerr << "Read info.\n";

		//Send message variables
		int messlen;
		int messstart;
		char destname[255];
		int leftinQ;

		//Get message variables
		char get_message[255];
		int get_message_len;

		//Interpret Command Here
		switch (buffer[0])
		{
		case 0x00:
			///Send Message
			// data format
			//  Ox00 [DESTINATION NAME] Ox00 [MESSAGE BYTES]
			
			/* Grab destination name, starts at &buffer[1] and is null terminated */
			strncpy(destname, &buffer[1], bytes);
			
			/* Calculate start of message and message length */
			messstart = strlen(destname) + 2;
			messlen = bytes-messstart;

			/* Post message to destinations queue */
			data->PostMess(destname, &buffer[messstart], messlen);

			//Generate response
			response[0] = 0;
			resplength=0;

			break;

		case 0x01:
			//GET Message
			leftinQ = data->GetMess(clientname, get_message, get_message_len);

			if(leftinQ >= 0)
			{
				response[0]=0;
				response[1]= leftinQ;
				int i;
				for(i=0;i<get_message_len; i++)
				{
					response[i+2] = get_message[i];
				}
				resplength=get_message_len+2;

			}
			else
			{
				response[0] = 0x01;
				resplength=1;
			}

			break;

		default:
			//unrecognized
			break;
		}
		

		

		
		if(WriteFile(pipe, response, resplength, &bytes, NULL) == 0)
		{
			//WriteFailed
		}

		Sleep(10);

	}

	return 0;
}