/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
///		CS460 - Christopher Kratz - Spring 2007                      ////
///                                                                  ////
///     Multi-Process Application Server                             ////
///                                                                  ////
///     Purpose:                                                     ////
///			This programm serves as a central mechanism for processes////
///		in a multi-process application to communicate with each other////
///		and act as a single system.                                  ////
///                                                                  ////
///		Modification Log:                                            ////
///			2-27-2007	C. Kratz	Created                          ////
///                                                                  ////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////


#include<afx.h>
#include<process.h>
#include<string.h>
#include<iostream.h>
#include<winbase.h>
#include"DataServer.h"		//cDataEngine class

const BUFFERSIZE = 255;		//Constant for size of general buffers



void StartProcess(char * name,char * pipename);			//Executes a process

HANDLE StartWorkerThread(char * clientname,				//Begins the worker thread
						 char * pipename, 
						 cDataEngine * data);

unsigned int _stdcall WorkerThread(void * data);		//Worker thread


struct WorkerThreadStartInfo
{
	cDataEngine * data;		
	char ClientName[255];
	char PipeName[255];
};

void main()
{

	/*	this class holds all the functionality of the data and messaging
		system. It is put in the for of a class so it can be passed easily
		to the worker threads when they are initiated.						*/
	cDataEngine test;


	/*	Array to hold handles of started processes. Right now this is configured
		to start the processes and wait for them to end before the main thread
		terminates. This functionality could be modified later to provide a way
		to monitor, stop, or restart worker thread.							*/
	HANDLE t[2];

	t[0] = StartWorkerThread("Agent1", "\\\\.\\pipe\\Agent1", &test);
	//Sleep(500);
	t[1] = StartWorkerThread("Agent2", "\\\\.\\pipe\\Agent2", &test);

	/* Wont return until both threads are in signaled state					*/
	WaitForMultipleObjects(2, t, true, INFINITE);
}


void StartProcess(char * name,char * pipename)
{
	/* Starts a process using win32 api CreateProcess funciton */

	_PROCESS_INFORMATION info;		// Structure CreateProcess uses to return information on
									//		the created process

	STARTUPINFO sinfo;				// Struture containing information on how to start the process
									//		required for CreateProcess

	GetStartupInfo(&sinfo);			// Uses this processes startup info for the new process


	char command[255];				// Builds a commmand line for the new process
	strcpy(command, "name");
	strcat(command, " ");
	strcat(command, pipename);

	if(CreateProcess(name, command, NULL, NULL, FALSE, 
						CREATE_NEW_CONSOLE, NULL, NULL, &sinfo, &info)==0)
	{
		//cout << "CreateProcess Failed...\n";
		return;
	}

}

HANDLE StartWorkerThread(char * clientname, char * pipename, cDataEngine * data)
{
	/* Begins the worker thread */

	/* IMPORTANT: the WorkerThreadStartInfo structure is uses to pass data
					to the worker thread, since using the beginthreadex function
					you can only specify a single void pointer as a parameter.
					Using dynamic memory is important, if you use a local variable
					to pass the info, you cannot garuantee that the worker thread
					will get the data. Local variables reside in the current stack
					frame, but once this function returns, that area in memory will
					be replaced by the next function called in the main thread and 
					will change the data that the worker thread can see.	*/

	WorkerThreadStartInfo * startinfo = new WorkerThreadStartInfo;

	/* Copy pipe name, client name, and data pointer into startinfo */
	strcpy(startinfo->ClientName, clientname);
	strcpy(startinfo->PipeName, pipename);
	startinfo->data = data;

	/*begin thread*/
	unsigned int thread;
	return (HANDLE)_beginthreadex(NULL, 0, WorkerThread, startinfo, 0, &thread);


}


unsigned int _stdcall WorkerThread(void * clientdata)
{
	/* This is the worker thread. It is responsible for maintaining the pipe
		interpreting and acting on commands it recieves from client processes */

	char clientname[255];
	char pipename[255];

	strcpy(clientname, ((WorkerThreadStartInfo*)clientdata)->ClientName);
	strcpy(pipename, ((WorkerThreadStartInfo*)clientdata)->PipeName);
	cDataEngine * data = ((WorkerThreadStartInfo*)clientdata)->data;

	/* This was dynamically created in StartWorkerThread, must deallocate */
	delete clientdata;


	/* Create the pipe. F1 on the function name for details of arguments. */
	HANDLE pipe		=	CreateNamedPipe(pipename,
										PIPE_ACCESS_DUPLEX,
										PIPE_TYPE_MESSAGE | PIPE_READMODE_BYTE,
										PIPE_UNLIMITED_INSTANCES,
										256,
										256,
										0,
										NULL);


	/* Debug info for console testing, must change later */
	cerr << "Connecting Pipe " << clientname << "\n";

	/* Wait for the client to connect */
	if( ConnectNamedPipe(pipe, NULL) == 0 )
	{	
		DWORD result = GetLastError();
		if(result != ERROR_PIPE_CONNECTED)
		{
			return 0;
		}
	}

	cerr << "Connected Pipe " << clientname << "\n";

	
	char response[BUFFERSIZE];		//Used to store response From Server
	int resplength;

	char buffer[255];
	unsigned long bytes=0;

	while(true)
	{
		if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
		{
			//Read Failed
			if(GetLastError()==ERROR_BROKEN_PIPE)
			{
				//Pipe Broken
				/* TODO: Add code to handle and try to recover from broken pipe
						 this may include but is not limited to, attempting to reopen the 
						 pipe, or restarting worker process. 
						 Note: This should be part of the system standard so the client knows
								how to recover from a broken pipe as well.
				*/
				return 0;
			}
			
		}

		cerr << "Read info.\n";

		//Send message variables
		int messlen;
		int messstart;
		char destname[255];
		int leftinQ;
		int ServiceCode;

		int cntDest;
		char (*Dests)[SIZE_AGENTNAME];
		int offset;
		int sent;

		//Get message variables
		char get_message[255];
		int get_message_len;

		char from[SIZE_AGENTNAME];

		//Interpret Command Here
		switch (buffer[0])
		{
		case 0x00:
			///Send Message
			// data format
			//  Ox00 [DESTINATION NAME] Ox00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]
			
			/* Grab destination name, starts at &buffer[1] and is null terminated */
			strncpy(destname, &buffer[1], bytes);

			/* Get Service Code From Message */
			ServiceCode = (int)buffer[strlen(destname) + 2] * 256;
			ServiceCode += (int)buffer[strlen(destname) + 3];
			
			/* Calculate start of message and message length */
			messstart = strlen(destname) + 4;
			messlen = bytes-messstart;

			/* Post message to destinations queue */
			if(!data->PostMess(destname, clientname, &buffer[messstart], messlen, ServiceCode))
			{
				/*Could not post message*/
				response[0] = 0x01;
				resplength = 1;
			}
			else
			{
				//Generate response
				response[0] = 0x00;
				resplength=1;
			}

			break;

		case 0x01:
			//GET Message
			// response data format
			// [RETURN CODE x2 BYTES] [REMAINING MESSAGES x2 BYTES] [SERVICE CODE x2 BYTES] [FROM] 0x00 [MESSAGE DATA]
			
			/* Retrieve the next message from the queue */
			leftinQ = data->GetMess(clientname, from, get_message, get_message_len, ServiceCode);

			if(leftinQ >= 0)
			{
				response[0]=0;						/* Response Code For Success is 0 */
				response[1]=0;						/* Return as 16-bit integer       */

				response[2]= (leftinQ / 256);		/* Return Number of messages left */
				response[3]= (leftinQ % 256);		/* Return as 16-bit integer       */

				response[4]= (ServiceCode / 256);	/* Return ServiceCode of message  */
				response[5]= (ServiceCode % 256);	/* Return as 16-bit integer		  */

				int len = strlen(from);

				/* Copy from into the response*/
				int i;
				for(i=0;i<len; i++)
				{
					response[i+6] = from[i];
				}

				/* Terminate From */
				response[i+6] = 0x00;
				int offset=len+1+6;

				/* Copy Data into response */
				for(i=0;i<get_message_len; i++)
				{
					response[i+offset] = get_message[i];
				}
				resplength=get_message_len+offset;

			}
			else
			{
				/* NO MESSAGES TO RETIRIEVE */
				/* Format: 0x00             */

				response[0] = 0x01;
				resplength=1;
			}

			break;
		case 0x02:
			///Send MultiCast Message Message
			// data format
			//  Ox02 [NUMBER OF DESTINATIONS x2 BYTES] [DESTINATION 1] Ox00 [DESTINATION 2] 0x00 .... [DESTNATION N] 0x00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]
			
			/* Get Number of destinations*/
			cntDest = (int)buffer[1] * 256;
			cntDest += (int)buffer[2];

			/* Destination Name array */
			Dests = new char[cntDest][SIZE_AGENTNAME];
			if(Dests == NULL)
			{
				/*FAILED TO GET MEMORY*/
				response[0] = 0x01;
				resplength=1;
				break;
			}
	
			int i;
			offset = 3;
			for(i=0;i<cntDest;i++)
			{
				/* Get destination name and increment offset */
				strncpy(Dests[i], &buffer[offset], SIZE_AGENTNAME);
				offset += strlen(Dests[i]) + 1;
			}


			/* Get Service Code From Message */
			ServiceCode = (int)buffer[strlen(destname) + offset] * 256;
			ServiceCode += (int)buffer[strlen(destname) + offset+1];
			offset += 2;

			/* Calculate start of message and message length */
			messstart = offset;
			messlen = bytes-messstart;
			
			sent=0;
			for(i=0;i<cntDest;i++)
			{
				
				/* Post message to destinations queue */
				if(!data->PostMess(Dests[i], clientname, &buffer[messstart], messlen, ServiceCode))
				{
					/*Could not post message*/
				}
				else
				{
					/*Message Sent*/
					sent++;
				}
			}

			if(sent == 0)
			{
				response[0] = 0x02;
				resplength  = 1;
			}
			else
			{
				response[0] = 0x00;
				response[1] = sent / 256;
				response[2] = sent % 256;
				resplength=3;
			}

			break;


		default:
			//unrecognized
			break;
		}
		

		
		/* Write the response to the pipe */		
		if(WriteFile(pipe, response, resplength, &bytes, NULL) == 0)
		{
			/* Add error handling for busted pipes */
			//WriteFailed
		}

		Sleep(10);

	}

	return 0;
}

/*####################################################################################*/
/*                                                                                    */
/*    %%%%  %%%%   %%%%%%%    %%%%%%%%%  %%%%%%%%%  %%%%%%%%%                         */
/*    %%%%  %%%%   %%%%%%%%   %%%   %%%  %%%%%%%%%  %%%%%%%%%                         */
/*   %%%%%%%%     %%%%  %%%  %%%%%%%%%     %%%        %%%%%                           */
/*   %%%%%%%%     %%%%%%%%   %%%%%%%%%     %%%      %%%%                              */
/*  %%%%  %%%%   %%%% %%%   %%%   %%%     %%%     %%%%%%%%%                           */
/*  %%%%  %%%%   %%%  %%%%  %%%   %%%     %%%     %%%%%%%%%                           */
/*                                                                                    */
/*      #######   ####    ####    ########       ****     ***     ***    ******       */
/*     #########  #####   ####   ##########     ******   *****   *****   *******      */
/*    ####        ######  ####  ####    ###          *  **   ** **   **       **      */
/*    ####        ######  ####  ####               **   **   ** **   **      **       */
/*     #######    ####### ####  ####              **    **   ** **   **     **        */
/*      #######   ############  ####            ******   *****   *****     **         */
/*          ####  #### #######  ####            ******    ***     ***     **          */
/*          ####  ####  ######  ####    ###                                           */
/*    #########   ####   #####   ##########      -- ---  !  ! !    +--+               */
/*     #######    ####    ####    ########      |   ",   +--! !--+ |  |               */
/*                                               -- --"     ! !__! +--+               */
/*####################################################################################*/